#!/usr/bin/env python# -*- coding: utf-8 -*-

import matplotlib.pyplot as plt
import cartopy.crs as ccrs
import xarray as xr
import numpy as np

# Load dataset with xarray
dataset = xr.open_dataset("eddy_trajectory_2.0exp_19930101_20191015.nc")

# Projection
proj=ccrs.PlateCarree()

#------------------------------------------------------------------------#
# Follow a given eddy, plot its filtered trajectory and the original one #
#------------------------------------------------------------------------#

# Select a specific eddy and save
subset = dataset.sel(obs=dataset.track==151011)
# Store the selected trajectory
subset.to_netcdf('eddy_trajectory_151011.nc')

# Filter the position with a rolling mean
windows_kwargs = dict(min_periods=7, obs=14, center=True)
new_lon = subset.longitude.rolling(** windows_kwargs).mean()
new_lat = subset.latitude.rolling(** windows_kwargs).mean()

# Create figure
fig = plt.figure(figsize=(5,2),dpi=96*2)
# Create subplot
ax = fig.add_subplot(111, projection=proj)
# Plot the two tracks
ax.plot(subset.longitude, subset.latitude, color='r', linewidth=2, label='Original', transform=proj)
ax.plot(new_lon, new_lat, color='b', label='Filtered', transform=proj)
# Add title
ax.set_title("Original and filtered trajectories", fontsize=10)
# Active meridian/parallel
ax.gridlines(draw_labels=True)
# Active coastline
ax.coastlines()
# Legend
ax.legend(loc=3)

# Display figure
plt.show()

#------------------------------------------------------------------------------------------#
# Select all eddies which go throught a given area and which have a lifespan over 500 days #
#------------------------------------------------------------------------------------------#


# Bounds of the area
lon_min, lon_max, lat_min, lat_max = 10, 20, -40, -34


# Select all observation in the area
subset = dataset.sel(
    obs=(dataset.longitude > lon_min) & (dataset.longitude < lon_max) & (dataset.latitude > lat_min) & (dataset.latitude < lat_max))
# Create a mask with all the trajectories passing throught the area
# Create the subset with the mask
subset = dataset.isel(obs=np.in1d(dataset.track, subset.track))
# Find all the trajecotries which are longer than 500 days
subset_lon_life =subset.sel(obs=subset.observation_number>500)
# Create the final subset
subset = subset.isel(obs=np.in1d(subset.track, subset_lon_life.track))

# Store subset to further analyse
subset.to_netcdf('eddy_trajectory_area_days_more500.nc')

# Create figure
fig = plt.figure(figsize=(5,3),dpi=96*2)
# Create subplot
ax = fig.add_subplot(111, projection=proj)

# Draw area
ax.fill(
    [lon_min, lon_max, lon_max, lon_min, lon_min],
    [lat_min, lat_min, lat_max, lat_max, lat_min],
    color='coral',
    transform=proj,
    alpha=0.4,
    zorder=0)

# Plot selected data
ax.scatter(
    subset.longitude,
    subset.latitude,
    c=subset.track,
    s=2,
    transform=proj,
    linewidth=0,
    cmap='tab20b')
# Add title
ax.set_title("Trajectories longer than 500 days", fontsize=10)
# Active meridian/parallel
ax.gridlines(draw_labels=True)
# Active coastline
ax.coastlines()
# Display figure
plt.show()

#------------------------------------------------#
# Select all eddies with an amplitude over 40 cm #
#------------------------------------------------#

# Selection of all eddies with an amplitude over 40 cm
subset = dataset.sel(obs=dataset.amplitude>0.40)
# Create the corresponding subset : trajectories with at least one eddy with Amp > 40 cm
subset = dataset.isel(obs=np.in1d(dataset.track, subset.track))

# Create figure
fig = plt.figure()
# Create subplot
ax = fig.add_subplot(111, projection=proj)

# Plot selected data
ax.scatter(
    subset.longitude,
    subset.latitude,
    c=subset.track,
    s=2,
    transform=proj,
    linewidth=0,
    cmap='magma')
# Add title
ax.set_title("Trajectories with Amplitude > 40 cm", fontsize=10)
# Active meridian/parallel
ax.gridlines(draw_labels=True)
# Active coastline
ax.coastlines()

# Display figure
plt.show()

# Save in a netcdf file 
subset.to_netcdf('eddy_trajectory_amplitude_more40.nc')



